@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: action.py 617 2005-02-27 03:03:05Z pronovic $
+# Revision : $Id: action.py 684 2005-03-08 19:14:43Z pronovic $
 # Purpose  : Provides implementation of various backup-related actions.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -84,7 +84,8 @@ from CedarBackup2.image import IsoImage
 from CedarBackup2.writer import CdWriter
 from CedarBackup2.writer import MEDIA_CDR_74, MEDIA_CDRW_74, MEDIA_CDR_80, MEDIA_CDRW_80
 from CedarBackup2.filesystem import BackupFileList, PurgeItemList, compareContents
-from CedarBackup2.util import executeCommand, getUidGid, getFunctionReference, deviceMounted
+from CedarBackup2.util import executeCommand, getUidGid, changeOwnership
+from CedarBackup2.util import getFunctionReference, deviceMounted, displayBytes
 from CedarBackup2.config import DEFAULT_DEVICE_TYPE, DEFAULT_MEDIA_TYPE
 
 
@@ -148,10 +149,10 @@ def _deriveDayOfWeek(dayName):
 
 
 ###########################
-# _todayIsStart() function
+# isStartOfWeek() function
 ###########################
 
-def _todayIsStart(startingDay):
+def isStartOfWeek(startingDay):
    """
    Indicates whether "today" is the backup starting day per configuration.
 
@@ -171,11 +172,11 @@ def _todayIsStart(startingDay):
    return value
 
 
-##################################
-# _buildNormalizedPath() function
-##################################
+#################################
+# buildNormalizedPath() function
+#################################
 
-def _buildNormalizedPath(absPath):
+def buildNormalizedPath(absPath):
    """
    Returns a "normalized" path based on an absolute path.
 
@@ -194,25 +195,6 @@ def _buildNormalizedPath(absPath):
    return normalized
 
 
-##############################
-# _changeOwnership() function
-##############################
-
-def _changeOwnership(path, user, group):
-   """
-   Changes ownership of path to match the user and group.
-   @param path: Path whose ownership to change.
-   @param user: User which owns file.
-   @param group: Group which owns file.
-   """
-   try:
-      (uid, gid) = getUidGid(user, group)
-      os.chown(path, uid, gid)
-   except Exception, e:
-      logger.error("Error changing ownership of [%s]: %s" % (path, e))
-
-
-
 ########################################################################
 # Attribute "getter" functions
 ########################################################################
@@ -288,7 +270,7 @@ def _getDigestPath(config, collectDir):
    @param collectDir: Collect directory object.
    @return: Absolute path to the digest associated with the collect directory.
    """
-   normalized = _buildNormalizedPath(collectDir.absolutePath)
+   normalized = buildNormalizedPath(collectDir.absolutePath)
    filename = "%s.%s" % (normalized, DIGEST_EXTENSION)
    digestPath = os.path.join(config.options.workingDir, filename)
    logger.debug("Digest path is [%s]" % digestPath)
@@ -313,7 +295,7 @@ def _getTarfilePath(config, collectDir, archiveMode):
       extension = "tar.gz"
    elif archiveMode == 'tarbz2':
       extension = "tar.bz2"
-   normalized = _buildNormalizedPath(collectDir.absolutePath)
+   normalized = buildNormalizedPath(collectDir.absolutePath)
    filename = "%s.%s" % (normalized, extension)
    tarfilePath = os.path.join(config.collect.targetDir, filename)
    logger.debug("Tarfile path is [%s]" % tarfilePath)
@@ -591,7 +573,7 @@ def executeCollect(configPath, options, config):
       raise ValueError("Collect configuration is not properly filled in.")
    fullBackup = options.full
    logger.debug("Full backup flag is [%s]" % fullBackup)
-   todayIsStart = _todayIsStart(config.options.startingDay)
+   todayIsStart = isStartOfWeek(config.options.startingDay)
    resetDigest = fullBackup or todayIsStart
    logger.debug("Reset digest flag is [%s]" % resetDigest)
    if config.collect.collectDirs is not None:
@@ -648,10 +630,10 @@ def _collectDirectory(config, absolutePath, tarfilePath, collectMode, archiveMod
    backupList.addDirContents(absolutePath)
    if collectMode != 'incr':
       logger.debug("Collect mode is [%s]; no digest will be used." % collectMode)
-      logger.info("Backing up %d files in this directory (%.0f bytes)." % (len(backupList), backupList.totalSize()))
+      logger.info("Backing up %d files in [%s] (%s)." % (len(backupList), absolutePath, displayBytes(backupList.totalSize())))
       if len(backupList) > 0:
          backupList.generateTarfile(tarfilePath, archiveMode, True)
-         _changeOwnership(tarfilePath, config.options.backupUser, config.options.backupGroup)
+         changeOwnership(tarfilePath, config.options.backupUser, config.options.backupGroup)
    else:
       if resetDigest:
          logger.debug("Based on resetDigest flag, digest will be cleared.")
@@ -662,10 +644,10 @@ def _collectDirectory(config, absolutePath, tarfilePath, collectMode, archiveMod
       newDigest = backupList.generateDigestMap()   # be sure to do this before removing unchanged files!
       removed = backupList.removeUnchanged(oldDigest)
       logger.debug("Removed %d unchanged files based on digest values." % removed)
-      logger.info("Backing up %d files in this directory (%.0f bytes)." % (len(backupList), backupList.totalSize()))
+      logger.info("Backing up %d files in [%s] (%s)." % (len(backupList), absolutePath, displayBytes(backupList.totalSize())))
       if len(backupList) > 0:
          backupList.generateTarfile(tarfilePath, archiveMode, True)
-         _changeOwnership(tarfilePath, config.options.backupUser, config.options.backupGroup)
+         changeOwnership(tarfilePath, config.options.backupUser, config.options.backupGroup)
       _writeDigest(config, newDigest, digestPath)
 
 def _loadDigest(digestPath):
@@ -705,7 +687,7 @@ def _writeDigest(config, digest, digestPath):
    """
    try: 
       pickle.dump(digest, open(digestPath, "w"))
-      _changeOwnership(digestPath, config.options.backupUser, config.options.backupGroup)
+      changeOwnership(digestPath, config.options.backupUser, config.options.backupGroup)
       logger.debug("Wrote new digest [%s] to disk: %d entries." % (digestPath, len(digest)))
    except: 
       logger.error("Failed to write digest [%s] to disk." % digestPath)
@@ -719,7 +701,7 @@ def _writeCollectIndicator(config):
    logger.debug("Writing collect indicator [%s]." % filename)
    try:
       open(filename, "w").write("")
-      _changeOwnership(filename, config.options.backupUser, config.options.backupGroup)
+      changeOwnership(filename, config.options.backupUser, config.options.backupGroup)
    except Exception, e:
       logger.error("Error writing collect indicator: %s", e)
 
@@ -774,7 +756,7 @@ def executeStage(configPath, options, config):
       logger.debug("Using target dir [%s], ownership [%d:%d]." % (targetDir, ownership[0], ownership[1]))
       try:
          count = peer.stagePeer(targetDir=targetDir, ownership=ownership)  # note: utilize effective user's default umask
-         logger.info("Staged %d files for this peer." % count)
+         logger.info("Staged %d files for peer [%s]." % (count, peer.name))
          peer.writeStageIndicator()
       except (ValueError, IOError, OSError), e:
          logger.error("Error staging [%s]: %s" % (peer.name, e))
@@ -803,7 +785,7 @@ def _createStagingDirs(config, dailyDir, peers):
          logger.debug("Creating staging directory [%s]." % dailyDir)
          os.makedirs(dailyDir)
          for path in [ dailyDir, os.path.join(dailyDir, ".."), os.path.join(dailyDir, "..", ".."), ]:
-            _changeOwnership(path, config.options.backupUser, config.options.backupGroup)
+            changeOwnership(path, config.options.backupUser, config.options.backupGroup)
       except Exception, e:
          raise Exception("Unable to create staging directory: %s" % e)
    for peer in peers:
@@ -815,7 +797,7 @@ def _createStagingDirs(config, dailyDir, peers):
          try:
             logger.debug("Creating peer staging directory [%s]." % peerDir)
             os.makedirs(peerDir)
-            _changeOwnership(peerDir, config.options.backupUser, config.options.backupGroup)
+            changeOwnership(peerDir, config.options.backupUser, config.options.backupGroup)
          except Exception, e:
             raise Exception("Unable to create staging directory: %s" % e)
    return mapping
@@ -836,7 +818,7 @@ def _writeStageIndicator(config, dailyDir):
    logger.debug("Writing stage indicator [%s]." % filename)
    try:
       open(filename, "w").write("")
-      _changeOwnership(filename, config.options.backupUser, config.options.backupGroup)
+      changeOwnership(filename, config.options.backupUser, config.options.backupGroup)
    except Exception, e:
       logger.error("Error writing stage indicator: %s", e)
 
@@ -874,7 +856,7 @@ def executeStore(configPath, options, config):
       raise ValueError("Store configuration is not properly filled in.")
    rebuildMedia = options.full
    logger.debug("Rebuild media flag [%s]" % rebuildMedia)
-   todayIsStart = _todayIsStart(config.options.startingDay)
+   todayIsStart = isStartOfWeek(config.options.startingDay)
    entireDisc = rebuildMedia or todayIsStart
    logger.debug("Entire disc flag [%s]" % entireDisc)
    stagingDirs = _findCorrectDailyDir(config)
@@ -950,19 +932,19 @@ def _writeImage(config, entireDisc, stagingDirs):
    @raise ValueError: Under many generic error conditions
    @raise IOError: If there is a problem writing the image to disc.
    """
-   logger.debug("entireDisc: %s" % entireDisc)
    writer = _getWriter(config)
    capacity = writer.retrieveCapacity(entireDisc=entireDisc)
-   logger.debug("Media capacity: %d bytes" % capacity.bytesAvailable)
+   logger.debug("Media capacity: %s" % displayBytes(capacity.bytesAvailable))
    image = IsoImage(writer.device, capacity.boundaries)
    for stageDir in stagingDirs.keys():
       logger.debug("Adding stage directory [%s]." % stageDir)
       dateSuffix = stagingDirs[stageDir]
       image.addEntry(path=stageDir, graftPoint=dateSuffix, contentsOnly=True)
    imageSize = image.getEstimatedSize()
-   logger.info("Image size will be %.0f bytes." % imageSize)
+   logger.info("Image size will be %s." % displayBytes(imageSize))
    if imageSize > capacity.bytesAvailable:
-      logger.error("Image (%.0f bytes) does not fit in available capacity (%.0f bytes)." % (imageSize, capacity.bytesAvailable))
+      logger.error("Image (%s) does not fit in available capacity (%s)." % (displayBytes(imageSize), 
+                                                                            displayBytes(capacity.bytesAvailable)))
       raise IOError("Media does not contain enough capacity to store image.")
    try:
       (handle, imagePath) = tempfile.mkstemp(dir=config.options.workingDir)
@@ -1041,7 +1023,7 @@ def _writeStoreIndicator(config, stagingDirs):
       logger.debug("Writing store indicator [%s]." % filename)
       try:
          open(filename, "w").write("")
-         _changeOwnership(filename, config.options.backupUser, config.options.backupGroup)
+         changeOwnership(filename, config.options.backupUser, config.options.backupGroup)
       except Exception, e:
          logger.error("Error writing store indicator: %s", e)
 
